from __future__ import annotations

import logging

import pytest

_ray = pytest.importorskip("ray")
from wandb_osh.ray_hooks import TriggerWandbSyncRayHook  # noqa: E402


class MockTrial:
    def __init__(self, logdir):
        self.logdir = logdir


def test_trigger_wandb_sync_hook(tmp_path, caplog):
    hook = TriggerWandbSyncRayHook(tmp_path)

    trial = MockTrial(logdir="/test/123")

    hook.log_trial_result(0, trial, {})  # type: ignore
    assert (tmp_path / "d2d46f.command").is_file()
    assert (tmp_path / "d2d46f.command").read_text() == "/test/123"
    with caplog.at_level(logging.WARNING):
        hook.log_trial_result(0, trial, {})  # type: ignore
    assert "Syncing not active or too slow" in caplog.text



from __future__ import annotations

import logging

from wandb_osh.hooks import TriggerWandbSyncHook


def test_trigger_wandb_sync_hook(tmp_path, caplog):
    hook = TriggerWandbSyncHook(tmp_path)
    hook("/test/123")
    assert (tmp_path / "d2d46f.command").is_file()
    assert (tmp_path / "d2d46f.command").read_text() == "/test/123"
    with caplog.at_level(logging.WARNING):
        hook("/test/123")
    assert "Syncing not active or too slow" in caplog.text



from __future__ import annotations

from os import PathLike
from pathlib import Path

import wandb

from wandb_osh import __version__
from wandb_osh.util.hash_id import hash_id
from wandb_osh.util.log import logger

_comm_default_dir = Path("~/.wandb_osh_command_dir").expanduser()


class TriggerWandbSyncHook:
    def __init__(self, communication_dir: PathLike = _comm_default_dir):
        """Hook to trigger synchronization of wandb with wandb-osh

        Args:
            communication_dir: Directory used for communication with wandb-osh.
        """
        self.communication_dir = Path(communication_dir)
        self.communication_dir.mkdir(parents=True, exist_ok=True)
        logger.info(
            "This is wandb-osh v%s using communication directory %s",
            __version__,
            self.communication_dir,
        )

    def __call__(self, logdir: str | PathLike | None = None):
        """Trigger synchronization on the head nodes

        Args:
            logdir: The directory in which wandb puts its run files.

        Returns:
            None
        """
        if logdir is None:
            # run.dir actually points to the `/files` subdirectory of the run,
            # but we need the directory above that.
            logdir = Path(wandb.run.dir).parent.resolve()
        trial_dir = Path(logdir).resolve()
        cmd_fname = hash_id(str(trial_dir)) + ".command"
        # In case the communication dir was deleted since we initialized this class
        self.communication_dir.mkdir(parents=True, exist_ok=True)
        command_file = self.communication_dir / cmd_fname
        if command_file.is_file():
            logger.warning(
                "Syncing not active or too slow: Command %s file still exists",
                command_file,
            )
        command_file.touch(exist_ok=True)
        command_file.write_text(trial_dir.resolve().as_posix())
        logger.debug("Wrote command file %s", command_file)



from __future__ import annotations

import logging

from wandb_osh.util.log import get_logger, logger
from wandb_osh import set_log_level


def test_logger():
    logger.info("Hello world")


def test_get_logger():
    assert isinstance(get_logger(), logging.Logger)


def test_set_log_level():
    set_log_level("ERROR")
    assert get_logger().level == logging.ERROR
    set_log_level()



from __future__ import annotations

from os import PathLike
from pathlib import Path

from ray.tune.experiment.trial import Trial
from ray.tune.logger import LoggerCallback

from wandb_osh.hooks import TriggerWandbSyncHook

_comm_default_dir = Path("~/.wandb_osh_command_dir").expanduser()


class TriggerWandbSyncRayHook(LoggerCallback):
    def __init__(self, communication_dir: PathLike = _comm_default_dir):
        """Hook to be used when interfacing wandb with ray tune.

        Args:
            communication_dir: Directory used for communication with wandb-osh.
        """
        super().__init__()
        self._hook = TriggerWandbSyncHook(communication_dir=communication_dir)

    def log_trial_result(self, iteration: int, trial: Trial, result: dict):
        trial_dir = Path(trial.logdir)
        self._hook(trial_dir)



from __future__ import annotations

from unittest.mock import Mock

import pytest
import wandb

_ray = pytest.importorskip("pytorch_lightning")

from wandb_osh.lightning_hooks import TriggerWandbSyncLightningCallback  # noqa: E402


def test_manual_trigger(tmp_path):
    wandb.init(project="test", mode="offline", dir=tmp_path)
    lh = TriggerWandbSyncLightningCallback(tmp_path)
    trainer = Mock()
    trainer.sanity_checking = False
    module = Mock()
    lh.on_validation_epoch_end(trainer, module)  # type: ignore
    assert len([f for f in tmp_path.iterdir() if f.suffix == ".command"]) == 1



from __future__ import annotations

import logging
import subprocess
import unittest
import unittest.mock
from pathlib import Path

from wandb_osh.syncer import WandbSyncer
from wandb_osh.util.log import set_log_level


def test_wandb_syncer(tmp_path, caplog):
    set_log_level("DEBUG")
    tmp_path = Path(tmp_path)
    ws = WandbSyncer(tmp_path)
    target = tmp_path / "test" / "123"
    (tmp_path / "123.command").write_text(str(target.resolve()))
    with caplog.at_level(logging.WARNING):
        ws.loop()
    assert "points to non-existing directory" in caplog.text
    caplog.clear()
    (tmp_path / "123.command").write_text(str(target.resolve()))
    target.mkdir(parents=True)
    with caplog.at_level(logging.DEBUG):
        ws.loop()
    assert f"Command would be: wandb sync . in {target.resolve()}" in caplog.text
    set_log_level()


def test_wandb_sync_timeout(tmp_path, caplog):
    with unittest.mock.patch(
        "wandb_osh.syncer.sync_dir", side_effect=subprocess.TimeoutExpired("asdf", 123)
    ):
        tmp_path = Path(tmp_path)
        ws = WandbSyncer(tmp_path)
        target = tmp_path / "test" / "123"
        (tmp_path / "123.command").write_text(str(target.resolve()))
        target.mkdir(parents=True)
        with caplog.at_level(logging.DEBUG):
            ws.loop()
        assert "timed out. Trying later." in caplog.text



# std
from __future__ import annotations

import logging

import colorlog

LOG_DEFAULT_LEVEL = logging.INFO


def get_logger():
    """Sets up global logger."""
    _log = colorlog.getLogger("wandb_osh")

    if _log.handlers:
        # the logger already has handlers attached to it, even though
        # we didn't add it ==> logging.get_logger got us an existing
        # logger ==> we don't need to do anything
        return _log

    _log.setLevel(LOG_DEFAULT_LEVEL)

    sh = colorlog.StreamHandler()
    log_colors = {
        "DEBUG": "cyan",
        "INFO": "green",
        "WARNING": "yellow",
        "ERROR": "red",
        "CRITICAL": "red",
    }
    formatter = colorlog.ColoredFormatter(
        "%(log_color)s%(asctime)s %(levelname)s: %(message)s",
        log_colors=log_colors,
        datefmt="%H:%M:%S",
    )
    sh.setFormatter(formatter)
    # Controlled by overall logger level
    sh.setLevel(logging.DEBUG)

    _log.addHandler(sh)

    return _log


def set_log_level(level: str | int = LOG_DEFAULT_LEVEL) -> None:
    """Sets the log level for the global logger."""
    logger = get_logger()
    logger.setLevel(level)


logger = get_logger()



from __future__ import annotations

from argparse import ArgumentParser

from wandb_osh.config import _command_dir_default
from wandb_osh.syncer import WandbSyncer


def _get_parser() -> ArgumentParser:
    parser = ArgumentParser(
        description="Wandb offline syncer. To pass arguments to wandb sync, e.g., "
        "`--sync-all`, use `--` before the arguments. For example: "
        "`wandb-osh --wait 10 -- --sync-all`"
    )
    parser.add_argument(
        "--command-dir",
        default=_command_dir_default,
        help="Command dir: Directory in which the hook creates files to trigger a "
        "synchronization.",
    )
    parser.add_argument(
        "--wait",
        default=1,
        type=float,
        help="Minimal time that has to pass before checking the command dir again.",
    )
    parser.add_argument(
        "--timeout",
        default=120,
        type=int,
        help="Timeout for wandb sync. If <=0, no timeout.",
    )
    parser.add_argument(
        "wandb_options",
        nargs="*",
        help="Options to be passed on to `wandb sync`, e.g. `--sync-all`. When "
        "specifying a flag to wandb, please add a `--` before the option, "
        "e.g., `wandb-osh -- --sync-all`",
    )
    return parser


def main(argv=None) -> None:
    parser = _get_parser()
    args = parser.parse_args(argv)
    wandb_osh = WandbSyncer(
        command_dir=args.command_dir,
        wait=args.wait,
        wandb_options=args.wandb_options,
        timeout=args.timeout,
    )
    wandb_osh.loop()


if __name__ == "__main__":
    main()



from __future__ import annotations

try:
    from importlib.metadata import version
except ModuleNotFoundError:
    from importlib_metadata import version  # type: ignore

__version__ = version("wandb_osh")


from wandb_osh.util.log import set_log_level


__all__ = ["__version__", "set_log_level"]
